/*
 * lmu.c - set keyboard backlight- and get environment light level
 * 
 * tested on: 
 * 	- PB 17" first model (PowerBook5,1)
 *
 *	Copyright (C) 2004 Alexander Clausen
 *	
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA 
 *
 */

#include <sys/types.h>
#include "i2c-dev.h"
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <stdlib.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>
#include <asm/types.h>
#include <errno.h>


#include <ctype.h>
#include <linux/fb.h>
#include <linux/pmu.h>


/* #define DEBUG */

#define ADDR 0x42

int
fblevel(int level)
{
        static int fd = -1;
        if(fd<0)
        {
                fd = open("/dev/pmu", O_RDONLY);
                if(fd<0)
                {
                       perror("/dev/pmu");
                       return 0;
                }
        }
        ioctl(fd, level<0?PMU_IOC_GET_BACKLIGHT:PMU_IOC_SET_BACKLIGHT, &level);

        return level;
}


int
open_bus ( ) {
	char filename[20], buf[4];
	int num = 0, file;

	/* try eacch bus */
	while ( num <= 16 ) {
		snprintf( filename, 19, "/dev/i2c-%d", num );
		
		if ( (file = open (filename, O_RDWR) ) < 0 ) {
			if ( errno == ENODEV ) {
				close ( file );
				return -1; /* last bus */
			} else if ( errno == EACCES ) {
				return -2; /* not root? */
			} else {
				num++; /* couldn't open file, try next bus */
				close ( file );
				continue;
			}
		}

		if ( ioctl ( file, I2C_SLAVE, ADDR) < 0 ) {
			num++; /* ioctl failed, try next bus */
			close ( file );
			continue;
		}
	
		
		if ( read ( file, buf, 4 ) == 4 ) {
			printf( "bus found: %s\n", filename );
			break; /* bus found */
		} {
			close ( file );
		}
		
		num++;
	}

	return ( file != -1 ) ? file : -1;
}

int
set_level ( int dev, unsigned short level ) {
	__u8 reg = 0x01;
	__u8 buf[3];
	
	/* check bounds */
	if ( level > 15 )
		return -1;
	
	buf[0] = reg;
	buf[1] = (__u8) (( ( level * 255 ) & 0xFF00 ) >> 8 );
	buf[2] = (__u8) (( ( level * 255 ) & 0x00FF ));
	
#ifdef DEBUG
	fprintf ( stderr, "0x%x\n",  ( ( level * 255 ) & 0xFF00 ) >> 8 );
	fprintf ( stderr, "0x%x\n",  ( ( level * 255 ) & 0x00FF ));
	fprintf ( stderr, "level=0x%04x buf[0]=0x%02x buf[1]=0x%02x buf[2]=0x%02x \n", 
			( level * 255 ), 
			(char)buf[0], (char)buf[1], (char)buf[2] );
#endif

	if ( write ( dev, buf, 3 ) != 3 ) {
		return -2;
	}

	return 1;
}

int
get_env_light_level ( int dev, int *left, int *right ) {
	char buf[4];

	if ( read ( dev, buf, 4 ) != 4 ) {
		return -1;
	}
	
#ifdef DEBUG
	fprintf ( stderr, "buf[0]=0x%02x buf[1]=0x%02x buf[2]=0x%02x buf[3]=0x%02x\n", buf[0], buf[1], buf[2], buf[3] );
#endif

	*right = ( buf[0] << 8 ) | buf[1];
	*left = ( buf[2] << 8 ) | buf[3];

	return 1;
}
