/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

/*
 *
 * Copyright Andreas Jaggi <andreas.jaggi@waterwav.ch>
 *
 * Based on ideas and lmu.c from Alexander Clausen
 *
 * Added support for seting automatically lcd brightness
 * by Federico Gamio <federico@gamio.net>
*/

#include "lmu.h"
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>

#define UPDATE_PERIOD 40 // time in ms between checks if light is off
#define CHANGE_PERIOD 5 // time in ms between scale to scale in fading in and out (like in OS/X)
#define LIGHT_THRESHOLD_KEYB 450
#define LIGHT_THRESHOLD_LCD 1500

int main ( ) {
	int left = 0, right = 0, dev;
	int level_lcd = 31, level_keyb = 0, new_keyb, new_lcd;

	dev = open_bus( );
	if (dev == -1) {
		printf("no lmu found\n");
		exit ( EXIT_FAILURE );
	}
	if (dev == -2) {
		printf("access denied\n");
		exit ( EXIT_FAILURE );
	}
	while(1) {
		if ( get_env_light_level ( dev, &left, &right ) == -1 ) {
			printf ( "read failed\n" );
			exit ( EXIT_FAILURE );
		}
		new_keyb = (-15*(right+left))/(2*LIGHT_THRESHOLD_KEYB) + 15;
		new_lcd = (29*(right+left))/(2*LIGHT_THRESHOLD_LCD) + 2;

		if ((left + right) > 2 * LIGHT_THRESHOLD_KEYB)
			new_keyb = 0;
		if ((left + right) > 2 * LIGHT_THRESHOLD_LCD)
			new_lcd = 31;
		while ((new_keyb != level_keyb) || (new_lcd != level_lcd)) {
			if (new_keyb != level_keyb) {
				level_keyb += new_keyb > level_keyb ? 1 : -1;
				set_level( dev, level_keyb);
			}
			if (new_lcd != level_lcd) {
//				printf ( "Luminosidad:%4d -> %4d\n", new_lcd, level_lcd );
				level_lcd += new_lcd > level_lcd ? 1 : -1;
				fblevel(level_lcd);
			}
			usleep(CHANGE_PERIOD * 10000L);
		}
		usleep(UPDATE_PERIOD * 10000L);
	}
	return 0;
}

